==> lori_sdk-0.7.0.tar.gz: lib/data/models/cart/cart.dart <==
import 'dart:convert';

import 'package:json_annotation/json_annotation.dart';
import 'package:lori_sdk/data/models/cart/cartItem.dart';
import 'package:lori_sdk/data/models/cart/freight.dart';
import 'package:lori_sdk/data/models/cart/paymentMethod.dart';
import 'package:lori_sdk/data/models/cart/seller.dart';
import 'package:lori_sdk/data/models/cart/sellerModality.dart';
import 'package:lori_sdk/data/models/cart/voucher.dart';
import 'package:lori_sdk/data/models/userAddress.dart';

part 'cart.g.dart';

final String tableName = 'cart';
final String columnId = 'id';
final String columnCpf = 'cpf';
final String columnDiscountValue = 'discount_value';
final String columnFreightValue = 'freight_value';
final String columnFeeValue = 'fee_value';
final String columnSubTotal = 'sub_total';
final String columnTotal = 'total';
final String columnSubTotalSeparated = 'sub_total_separated';
final String columnTotalSeparated = 'total_separated';
final String columnMaxInstallments = 'max_installments';
final String columnInstallments = 'installments';
final String columnFreight = 'freight';
final String columnDeliveryAddress = 'delivery_address';
final String columnIsDelivery = 'is_delivery';
final String columnDeliveryTime = 'delivery_time';
final String columnLatitude = 'latitude';
final String columnLongitude = 'longitude';
final String columnCep = 'cep';
final String columnCpfInvoice = 'cpf_invoice';
final String columnCpfPickup = 'cpf_pickup';
final String columnModality = 'modality';
final String columnItems = 'items';
final String columnSeller = 'seller';
final String columnPaymentMethod = 'payment_method';
final String columnSupportedPaymentMethods = 'supported_payment_methods';
final String columnVoucher = 'voucher';
final String columnIsCvvValidated = 'is_cvv_validated';
final String columnHasFeeInstallment = 'has_fee_installment';
final String columnGiftsQuantityAllowed = 'gifts_quantity_allowed';
final String columnAvailableGiftItems = 'available_gift_items';
final String columnChangesFromLastUpdate = 'changes_last_update';
final String columnMessages = "messages";

@JsonSerializable()
class Cart {
  Cart();

  String id;
  String cpf;

  double discountAmount;
  double freightAmount;
  double feeAmount;
  double subTotalAmount;
  double totalAmount;
  double subTotalSeparatedAmount;
  double totalSeparatedAmount;

  int maxInstallments;
  int installments;

  Freight freight;
  UserAddress deliveryAddress;
  bool isDelivery;
  int deliveryTime;

  double latitude;
  double longitude;

  String cep;

  String cpfInvoice;
  String cpfPickUp;

  SellerModality modality;

  @JsonKey(defaultValue: 0)
  double bonusAmount;

  /// We have three type of items on the cart
  ///
  ///
  /// loriItem: A Regular cartItem added explicitly by the user to the cart
  ///
  /// promotionalItem: A promotional cartItem added to the cart by the retailer
  ///   the promotionalItem MIGHT BE FREE but might only have a discount
  ///   it is NOT editable
  ///
  /// giftItem: A GIFT cartItem added to the cart by the retailer
  ///   the giftItem is always free since its a gift
  ///   it is editable through special endpoints

  /// All the cartItems returned by the API
  @JsonKey(name: 'items')
  List<CartItem> allItems;

  @Deprecated('Please use one of the specific items properties or use `allItems` if you really want them all.')
  @JsonKey(ignore: true)
  List<CartItem> get items {
    return this.allItems;
  }

  /// Shorthand property to get only the regular items
  @JsonKey(ignore: true)
  List<CartItem> get loriItems {
    return this.allItems.where((i) => !i.isPromotional && !i.isGift).toList();
  }

  /// Shorthand property to get only the promotional items
  @JsonKey(ignore: true)
  List<CartItem> get promotionalItems {
    return this.allItems.where((i) => i.isPromotional).toList();
  }

  /// Shorthand property to get only the regular AND promotional items
  ///   (at the cart we show only those two items)
  @JsonKey(ignore: true)
  List<CartItem> get cartItems {
    return this.allItems.where((i) => !i.isGift).toList();
  }

  /// Shorthand property to get only the gift items
  @JsonKey(ignore: true)
  List<CartItem> get giftItems {
    return this.allItems.where((i) => i.isGift).toList();
  }

  /// Shorthand property to get all separated items
  @JsonKey(ignore: true)
  List<CartItem> get separatedItems {
    return this.allItems.where((i) => i.quantitySeparated > 0).toList();
  }

  List<String> messages;

  @JsonKey(ignore: true)
  String get allMessages {
    String _allMessages = "";
    this.messages.forEach((m) {
      _allMessages += messages.length <= 1 ? m : "$m\n\n";
    });
    return _allMessages;
  }

  Seller seller;
  PaymentMethod paymentMethod;

  List<PaymentMethodType> supportedPaymentMethods;

  //Added this getter as a workaround to support a default value
  //If the API DOES NOT return a supportedPaymentMethods list on the cart the default will be [.credit]
  //But for some reason dart did not allow us to give a list on the defaultValue of the JsonKey...
  List<PaymentMethodType> get supportedPaymentTypes {
    return supportedPaymentMethods == null ? [PaymentMethodType.credit] : supportedPaymentMethods;
  }

  set supportedPaymentTypes(List<PaymentMethodType> methods) {
    if (methods != null) {
      supportedPaymentMethods = methods;
    }
  }

  Voucher voucher;

  bool isCvvValidated;
  bool hasFeeInstallment = false;

  // Gifts
  @JsonKey(defaultValue: 0)
  int giftsQuantityAllowed;

  List<CartItem> availableGiftItems = List<CartItem>();

  /// The map containing the changes on the lastUpdate
  ///
  /// This map will be updated at `isUpdatingFrom`, so this means that
  ///   at the second update it will already be lost
  @JsonKey(ignore: true)
  Map<String, dynamic> changesFromLastUpdate = {};

  // When the cart is updated there are some values we must rescue from the old cart object
  void isUpdatingFrom(Cart cart, {bool forceUpdate = false}) {
    this.allItems.forEach((cartItem) {
      // Check if this cart item exists on the already saved cart
      // The same sku can exist in two cart items IF one of them is promotional or gift,
      //   so we also compare if the flags are equal
      var previousCartItem = cart.allItems.firstWhere((i) => i.sku.id == cartItem.sku.id && i.isPromotional == cartItem.isPromotional && i.isGift == cartItem.isGift, orElse: () => null);

      if (previousCartItem != null) {
        // If it exists, call the update handler
        cartItem.isUpdatingFrom(previousCartItem, forceUpdate: forceUpdate);
      }
    });

    if (this.giftsQuantityAllowed != cart.giftsQuantityAllowed) {
      changesFromLastUpdate['giftsQuantityAllowed'] = cart.giftsQuantityAllowed;
    }
  }

  /// Some shorthand methods to simplify the check of `changesFromLastUpdate`
  ///   for the most used variables
  ///
  bool get hasChanged {
    return allItems.where((ci) => ci.hasChanged).length > 0 || changesFromLastUpdate.keys.length > 0;
  }

  bool get justReceivedGift {
    if (changesFromLastUpdate.containsKey('giftsQuantityAllowed') && this.giftsQuantityAllowed > 0) {
      return true;
    }

    return false;
  }

  // Method to simplify the validation of a cart
  bool get needsReload {
    bool reload = false;

    // Validate the cart expiration date
    final now = DateTime.now();
    if (this.expirationDate != null && now.isAfter(this.expirationDate)) {
      // The cart has expired, reload it
      return true;
    }

    // TODO: Further validation

    return reload;
  }

  /// The Cart has an expiration date
  ///
  /// The cart expiration date is the earliest item expiration date
  DateTime get expirationDate {
    // An empty cart never expires
    if (allItems.isEmpty) return null;

    // Get all items expiration dates
    var itemsExpirationDate = this.allItems.map((ci) => ci.expirationDate).toList();

    //Sort them
    itemsExpirationDate.sort((ldt, rdt) => ldt.compareTo(rdt));

    //Get the earlies expiration date
    return itemsExpirationDate.first;
  }

  static DateTime get defaultExpirationDate {
    return DateTime.now().add(Cart.timeToLive);
  }

  static Duration get timeToLive {
    return Duration(minutes: 30);
  }

  // JSON Factories
  factory Cart.fromJson(Map<String, dynamic> json) {
    var cart = _$CartFromJson(json);
    cart.allItems.sort((li, ri) => li.compareTo(ri));
    return cart;
  }

  Map<String, dynamic> toJson() => _$CartToJson(this);

  // Helpers method to support SQLite
  Map<String, dynamic> toMap() {
    var map = <String, dynamic>{
      columnId: id,
      columnCpf: cpf,
      columnDiscountValue: discountAmount,
      columnFreightValue: freightAmount,
      columnFeeValue: feeAmount,
      columnSubTotal: subTotalAmount,
      columnTotal: totalAmount,
      columnSubTotalSeparated: subTotalSeparatedAmount,
      columnTotalSeparated: totalSeparatedAmount,
      columnMaxInstallments: maxInstallments,
      columnInstallments: installments,
      columnFreight: json.encode(freight),
      columnDeliveryAddress: json.encode(deliveryAddress),
      columnIsDelivery: isDelivery ? 1 : 0,
      columnDeliveryTime: deliveryTime,
      columnLatitude: latitude,
      columnLongitude: longitude,
      columnCep: cep,
      columnCpfInvoice: cpfInvoice,
      columnCpfPickup: cpfPickUp,
      columnModality: json.encode(modality),
      columnItems: json.encode(allItems),
      columnSeller: json.encode(seller),
      columnPaymentMethod: json.encode(paymentMethod),
      columnSupportedPaymentMethods: json.encode(supportedPaymentTypes.map((pm) => pm.index).toList()),
      columnVoucher: json.encode(voucher),
      columnIsCvvValidated: isCvvValidated ? 1 : 0,
      columnHasFeeInstallment: hasFeeInstallment ? 1 : 0,
      columnGiftsQuantityAllowed: giftsQuantityAllowed,
      columnAvailableGiftItems: json.encode(availableGiftItems),
      columnChangesFromLastUpdate: json.encode(changesFromLastUpdate),
      columnMessages: json.encode(messages)
    };
    return map;
  }

  void fromMap(Map<String, dynamic> map) {
    id = map[columnId];
    cpf = map[columnCpf];
    discountAmount = map[columnDiscountValue];
    freightAmount = map[columnFreightValue];
    feeAmount = map[columnFeeValue];
    subTotalAmount = map[columnSubTotal];
    totalAmount = map[columnTotal];
    subTotalSeparatedAmount = map[columnSubTotalSeparated];
    totalSeparatedAmount = map[columnTotalSeparated];
    maxInstallments = map[columnMaxInstallments];
    installments = map[columnInstallments];
    isDelivery = map[columnIsDelivery] == 1 ? true : false;
    deliveryTime = map[columnDeliveryTime];
    latitude = map[columnLatitude];
    longitude = map[columnLongitude];
    cep = map[columnCep];
    cpfInvoice = map[columnCpfInvoice];
    cpfPickUp = map[columnCpfPickup];

    // Custom objects fetching
    Iterable iItems = json.decode(map[columnItems]);
    List<CartItem> cartItems = [];
    iItems.forEach((map) {
      CartItem item = CartItem.fromJson(map);
      item.migrateToCurrentVersion();
      cartItems.add(item);
    });
    this.allItems = cartItems;

    List<String> listOfMessages = [];
    if (map[columnMessages] != null) {
      Iterable cm = json.decode(map[columnMessages]);
      cm.forEach((message) {
        listOfMessages.add(message);
      });
    }
    this.messages = listOfMessages;

    modality = (map[columnModality] == null || map[columnModality] == 'null') ? null : SellerModality.fromJson(json.decode(map[columnModality]));
    freight = (map[columnFreight] == null || map[columnFreight] == 'null') ? null : Freight.fromJson(json.decode(map[columnFreight]));
    deliveryAddress = (map[columnDeliveryAddress] == null || map[columnDeliveryAddress] == 'null') ? null : UserAddress.fromJson(json.decode(map[columnDeliveryAddress]));
    seller = (map[columnSeller] == null || map[columnSeller] == 'null') ? null : Seller.fromJson(json.decode(map[columnSeller]));
    paymentMethod = (map[columnPaymentMethod] == null || map[columnPaymentMethod] == 'null') ? null : PaymentMethod.fromJson(json.decode(map[columnPaymentMethod]));

    if (map[columnSupportedPaymentMethods] != null) {
      Iterable pItems = json.decode(map[columnSupportedPaymentMethods]);
      List<PaymentMethodType> supportedPayments = [];
      if (pItems != null) {
        pItems.forEach((map) {
          supportedPayments.add(PaymentMethodType.values[map]);
        });
      }
      supportedPaymentTypes = supportedPayments;
    }

    voucher = map[columnVoucher] == 'null' ? null : Voucher.fromJson(json.decode(map[columnVoucher]));
    isCvvValidated = map[columnIsCvvValidated] == 1 ? true : false;
    hasFeeInstallment = map[columnHasFeeInstallment] == 1 ? true : false;

    giftsQuantityAllowed = map[columnGiftsQuantityAllowed];
    if (map[columnAvailableGiftItems] != null) {
      Iterable gItems = json.decode(map[columnAvailableGiftItems]);
      List<CartItem> giftItems = [];
      if (gItems != null) {
        gItems.forEach((map) {
          CartItem item = CartItem.fromJson(map);
          giftItems.add(item);
        });
      }
      availableGiftItems = giftItems;
    }

    changesFromLastUpdate = json.decode(map[columnChangesFromLastUpdate] ?? '{}');
  }
}

==> lori_sdk-0.7.3.tar.gz: lib/data/models/cart/cart.dart <==
import 'dart:convert';

import 'package:json_annotation/json_annotation.dart';
import 'package:lori_sdk/data/models/cart/cartItem.dart';
import 'package:lori_sdk/data/models/cart/freight.dart';
import 'package:lori_sdk/data/models/cart/paymentMethod.dart';
import 'package:lori_sdk/data/models/cart/seller.dart';
import 'package:lori_sdk/data/models/cart/sellerModality.dart';
import 'package:lori_sdk/data/models/cart/voucher.dart';
import 'package:lori_sdk/data/models/userAddress.dart';

part 'cart.g.dart';

final String tableName = 'cart';
final String columnId = 'id';
final String columnCpf = 'cpf';
final String columnDiscountValue = 'discount_value';
final String columnFreightValue = 'freight_value';
final String columnFeeValue = 'fee_value';
final String columnSubTotal = 'sub_total';
final String columnTotal = 'total';
final String columnSubTotalSeparated = 'sub_total_separated';
final String columnTotalSeparated = 'total_separated';
final String columnMaxInstallments = 'max_installments';
final String columnInstallments = 'installments';
final String columnFreight = 'freight';
final String columnDeliveryAddress = 'delivery_address';
final String columnIsDelivery = 'is_delivery';
final String columnDeliveryTime = 'delivery_time';
final String columnLatitude = 'latitude';
final String columnLongitude = 'longitude';
final String columnCep = 'cep';
final String columnCpfInvoice = 'cpf_invoice';
final String columnCpfPickup = 'cpf_pickup';
final String columnModality = 'modality';
final String columnItems = 'items';
final String columnSeller = 'seller';
final String columnPaymentMethod = 'payment_method';
final String columnSupportedPaymentMethods = 'supported_payment_methods';
final String columnVoucher = 'voucher';
final String columnIsCvvValidated = 'is_cvv_validated';
final String columnHasFeeInstallment = 'has_fee_installment';
final String columnGiftsQuantityAllowed = 'gifts_quantity_allowed';
final String columnAvailableGiftItems = 'available_gift_items';
final String columnChangesFromLastUpdate = 'changes_last_update';
final String columnMessages = "messages";
final String columnIsPickupOnly = "isPickupOnly";

@JsonSerializable()
class Cart {
  Cart();

  String id;
  String cpf;

  double discountAmount;
  double freightAmount;
  double feeAmount;
  double subTotalAmount;
  double totalAmount;
  double subTotalSeparatedAmount;
  double totalSeparatedAmount;

  int maxInstallments;
  int installments;

  Freight freight;
  UserAddress deliveryAddress;
  bool isDelivery;
  int deliveryTime;

  double latitude;
  double longitude;

  String cep;

  String cpfInvoice;
  String cpfPickUp;

  SellerModality modality;

  @JsonKey(defaultValue: false)
  bool isPickupOnly;


  @JsonKey(defaultValue: 0)
  double bonusAmount;

  /// We have three type of items on the cart
  ///
  ///
  /// loriItem: A Regular cartItem added explicitly by the user to the cart
  ///
  /// promotionalItem: A promotional cartItem added to the cart by the retailer
  ///   the promotionalItem MIGHT BE FREE but might only have a discount
  ///   it is NOT editable
  ///
  /// giftItem: A GIFT cartItem added to the cart by the retailer
  ///   the giftItem is always free since its a gift
  ///   it is editable through special endpoints

  /// All the cartItems returned by the API
  @JsonKey(name: 'items')
  List<CartItem> allItems;

  @Deprecated('Please use one of the specific items properties or use `allItems` if you really want them all.')
  @JsonKey(ignore: true)
  List<CartItem> get items {
    return this.allItems;
  }

  /// Shorthand property to get only the regular items
  @JsonKey(ignore: true)
  List<CartItem> get loriItems {
    return this.allItems.where((i) => !i.isPromotional && !i.isGift).toList();
  }

  /// Shorthand property to get only the promotional items
  @JsonKey(ignore: true)
  List<CartItem> get promotionalItems {
    return this.allItems.where((i) => i.isPromotional).toList();
  }

  /// Shorthand property to get only the regular AND promotional items
  ///   (at the cart we show only those two items)
  @JsonKey(ignore: true)
  List<CartItem> get cartItems {
    return this.allItems.where((i) => !i.isGift).toList();
  }

  /// Shorthand property to get only the gift items
  @JsonKey(ignore: true)
  List<CartItem> get giftItems {
    return this.allItems.where((i) => i.isGift).toList();
  }

  /// Shorthand property to get all separated items
  @JsonKey(ignore: true)
  List<CartItem> get separatedItems {
    return this.allItems.where((i) => i.quantitySeparated > 0).toList();
  }

  List<String> messages;

  @JsonKey(ignore: true)
  String get allMessages {
    String _allMessages = "";
    this.messages.forEach((m) {
      _allMessages += messages.length <= 1 ? m : "$m\n\n";
    });
    return _allMessages;
  }

  Seller seller;
  PaymentMethod paymentMethod;

  List<PaymentMethodType> supportedPaymentMethods;

  //Added this getter as a workaround to support a default value
  //If the API DOES NOT return a supportedPaymentMethods list on the cart the default will be [.credit]
  //But for some reason dart did not allow us to give a list on the defaultValue of the JsonKey...
  List<PaymentMethodType> get supportedPaymentTypes {
    return supportedPaymentMethods == null ? [PaymentMethodType.credit] : supportedPaymentMethods;
  }

  set supportedPaymentTypes(List<PaymentMethodType> methods) {
    if (methods != null) {
      supportedPaymentMethods = methods;
    }
  }

  Voucher voucher;

  bool isCvvValidated;
  bool hasFeeInstallment = false;

  // Gifts
  @JsonKey(defaultValue: 0)
  int giftsQuantityAllowed;

  List<CartItem> availableGiftItems = List<CartItem>();

  /// The map containing the changes on the lastUpdate
  ///
  /// This map will be updated at `isUpdatingFrom`, so this means that
  ///   at the second update it will already be lost
  @JsonKey(ignore: true)
  Map<String, dynamic> changesFromLastUpdate = {};

  // When the cart is updated there are some values we must rescue from the old cart object
  void isUpdatingFrom(Cart cart, {bool forceUpdate = false}) {
    this.allItems.forEach((cartItem) {
      // Check if this cart item exists on the already saved cart
      // The same sku can exist in two cart items IF one of them is promotional or gift,
      //   so we also compare if the flags are equal
      var previousCartItem = cart.allItems.firstWhere((i) => i.sku.id == cartItem.sku.id && i.isPromotional == cartItem.isPromotional && i.isGift == cartItem.isGift, orElse: () => null);

      if (previousCartItem != null) {
        // If it exists, call the update handler
        cartItem.isUpdatingFrom(previousCartItem, forceUpdate: forceUpdate);
      }
    });

    if (this.giftsQuantityAllowed != cart.giftsQuantityAllowed) {
      changesFromLastUpdate['giftsQuantityAllowed'] = cart.giftsQuantityAllowed;
    }
  }

  /// Some shorthand methods to simplify the check of `changesFromLastUpdate`
  ///   for the most used variables
  ///
  bool get hasChanged {
    return allItems.where((ci) => ci.hasChanged).length > 0 || changesFromLastUpdate.keys.length > 0;
  }

  bool get justReceivedGift {
    if (changesFromLastUpdate.containsKey('giftsQuantityAllowed') && this.giftsQuantityAllowed > 0) {
      return true;
    }

    return false;
  }

  // Method to simplify the validation of a cart
  bool get needsReload {
    bool reload = false;

    // Validate the cart expiration date
    final now = DateTime.now();
    if (this.expirationDate != null && now.isAfter(this.expirationDate)) {
      // The cart has expired, reload it
      return true;
    }

    // TODO: Further validation

    return reload;
  }

  /// The Cart has an expiration date
  ///
  /// The cart expiration date is the earliest item expiration date
  DateTime get expirationDate {
    // An empty cart never expires
    if (allItems.isEmpty) return null;

    // Get all items expiration dates
    var itemsExpirationDate = this.allItems.map((ci) => ci.expirationDate).toList();

    //Sort them
    itemsExpirationDate.sort((ldt, rdt) => ldt.compareTo(rdt));

    //Get the earlies expiration date
    return itemsExpirationDate.first;
  }

  static DateTime get defaultExpirationDate {
    return DateTime.now().add(Cart.timeToLive);
  }

  static Duration get timeToLive {
    return Duration(minutes: 30);
  }

  // JSON Factories
  factory Cart.fromJson(Map<String, dynamic> json) {
    var cart = _$CartFromJson(json);
    cart.allItems.sort((li, ri) => li.compareTo(ri));
    return cart;
  }

  Map<String, dynamic> toJson() => _$CartToJson(this);

  // Helpers method to support SQLite
  Map<String, dynamic> toMap() {
    var map = <String, dynamic>{
      columnId: id,
      columnCpf: cpf,
      columnDiscountValue: discountAmount,
      columnFreightValue: freightAmount,
      columnFeeValue: feeAmount,
      columnSubTotal: subTotalAmount,
      columnTotal: totalAmount,
      columnSubTotalSeparated: subTotalSeparatedAmount,
      columnTotalSeparated: totalSeparatedAmount,
      columnMaxInstallments: maxInstallments,
      columnInstallments: installments,
      columnFreight: json.encode(freight),
      columnDeliveryAddress: json.encode(deliveryAddress),
      columnIsDelivery: isDelivery ? 1 : 0,
      columnDeliveryTime: deliveryTime,
      columnLatitude: latitude,
      columnLongitude: longitude,
      columnCep: cep,
      columnCpfInvoice: cpfInvoice,
      columnCpfPickup: cpfPickUp,
      columnModality: json.encode(modality),
      columnItems: json.encode(allItems),
      columnSeller: json.encode(seller),
      columnPaymentMethod: json.encode(paymentMethod),
      columnSupportedPaymentMethods: json.encode(supportedPaymentTypes.map((pm) => pm.index).toList()),
      columnVoucher: json.encode(voucher),
      columnIsCvvValidated: isCvvValidated ? 1 : 0,
      columnHasFeeInstallment: hasFeeInstallment ? 1 : 0,
      columnGiftsQuantityAllowed: giftsQuantityAllowed,
      columnAvailableGiftItems: json.encode(availableGiftItems),
      columnChangesFromLastUpdate: json.encode(changesFromLastUpdate),
      columnMessages: json.encode(messages),
      columnIsPickupOnly: isPickupOnly ? 1 : 0,
    };
    return map;
  }

  void fromMap(Map<String, dynamic> map) {
    id = map[columnId];
    cpf = map[columnCpf];
    discountAmount = map[columnDiscountValue];
    freightAmount = map[columnFreightValue];
    feeAmount = map[columnFeeValue];
    subTotalAmount = map[columnSubTotal];
    totalAmount = map[columnTotal];
    subTotalSeparatedAmount = map[columnSubTotalSeparated];
    totalSeparatedAmount = map[columnTotalSeparated];
    maxInstallments = map[columnMaxInstallments];
    installments = map[columnInstallments];
    isDelivery = map[columnIsDelivery] == 1 ? true : false;
    deliveryTime = map[columnDeliveryTime];
    latitude = map[columnLatitude];
    longitude = map[columnLongitude];
    cep = map[columnCep];
    cpfInvoice = map[columnCpfInvoice];
    cpfPickUp = map[columnCpfPickup];
    isPickupOnly = map[columnIsPickupOnly] == 1 ? true : false;

    // Custom objects fetching
    Iterable iItems = json.decode(map[columnItems]);
    List<CartItem> cartItems = [];
    iItems.forEach((map) {
      CartItem item = CartItem.fromJson(map);
      item.migrateToCurrentVersion();
      cartItems.add(item);
    });
    this.allItems = cartItems;

    List<String> listOfMessages = [];
    if (map[columnMessages] != null) {
      Iterable cm = json.decode(map[columnMessages]);
      cm.forEach((message) {
        listOfMessages.add(message);
      });
    }
    this.messages = listOfMessages;

    modality = (map[columnModality] == null || map[columnModality] == 'null') ? null : SellerModality.fromJson(json.decode(map[columnModality]));
    freight = (map[columnFreight] == null || map[columnFreight] == 'null') ? null : Freight.fromJson(json.decode(map[columnFreight]));
    deliveryAddress = (map[columnDeliveryAddress] == null || map[columnDeliveryAddress] == 'null') ? null : UserAddress.fromJson(json.decode(map[columnDeliveryAddress]));
    seller = (map[columnSeller] == null || map[columnSeller] == 'null') ? null : Seller.fromJson(json.decode(map[columnSeller]));
    paymentMethod = (map[columnPaymentMethod] == null || map[columnPaymentMethod] == 'null') ? null : PaymentMethod.fromJson(json.decode(map[columnPaymentMethod]));

    if (map[columnSupportedPaymentMethods] != null) {
      Iterable pItems = json.decode(map[columnSupportedPaymentMethods]);
      List<PaymentMethodType> supportedPayments = [];
      if (pItems != null) {
        pItems.forEach((map) {
          supportedPayments.add(PaymentMethodType.values[map]);
        });
      }
      supportedPaymentTypes = supportedPayments;
    }

    voucher = map[columnVoucher] == 'null' ? null : Voucher.fromJson(json.decode(map[columnVoucher]));
    isCvvValidated = map[columnIsCvvValidated] == 1 ? true : false;
    hasFeeInstallment = map[columnHasFeeInstallment] == 1 ? true : false;

    giftsQuantityAllowed = map[columnGiftsQuantityAllowed];
    if (map[columnAvailableGiftItems] != null) {
      Iterable gItems = json.decode(map[columnAvailableGiftItems]);
      List<CartItem> giftItems = [];
      if (gItems != null) {
        gItems.forEach((map) {
          CartItem item = CartItem.fromJson(map);
          giftItems.add(item);
        });
      }
      availableGiftItems = giftItems;
    }

    changesFromLastUpdate = json.decode(map[columnChangesFromLastUpdate] ?? '{}');
  }
}
